#!/usr/bin/env python
# vim: set fileencoding=UTF-8 :

#Libraries
from l_config import config
import l_analog
import l_drive

from datetime import datetime
import time
from multiprocessing import Process, Value
from ctypes import c_double,c_int

class odo_asynch:
    def __init__(self):
        self.t_label=Value(c_double, 0.0)
        self.enc_l = Value(c_int, 0)
        self.enc_r= Value(c_int, 0)
        self.sgn_left=Value(c_int,0)
        self.sgn_right=Value(c_int,0)
        
    def start(self,f):    
        self.p = Process(target=self.encoders, args=(self.enc_l,self.enc_r,self.t_label,self.sgn_left,self.sgn_right,f))
        self.p.start()

    def kill(self):
        self.p.terminate()

    
    def set_sign(self,signs):
        self.sgn_left.value=signs[0]
        self.sgn_right.value=signs[1]
        
        
    def encoders(self,c_l,c_r,t,left_sign,right_sign,f=False):
        
        p_l=l_analog.mpc3008(config.odo.left_channel)
        p_r=l_analog.mpc3008(config.odo.right_channel)
        border=config.odo.switch_border
        
        

        #Initial states
        v_l=float(p_l.read_data())/1024.
        v_r=float(p_r.read_data())/1024.
        enc_l_flag=True if v_l>border else False
        enc_r_flag=True if v_r>border else False
        
        start = datetime.now()
        

        
        while True:
            now = datetime.now()
            delta = now - start
            s = delta.seconds + delta.microseconds / 1E6
            t.value = s
            #print s
            #Odometry
            v_l=float(p_l.read_data())/1024.
            v_r=float(p_r.read_data())/1024.

            #print "Left ",left_sign.value," Right ",right_sign.value
            
            if (enc_l_flag and v_l<border) or (not enc_l_flag and v_l>border):        
                c_l.value+=1*left_sign.value
                enc_l_flag=not enc_l_flag       
                
            if (enc_r_flag and v_r<border) or (not enc_r_flag and v_r>border):        
                c_r.value+=1*right_sign.value
                enc_r_flag=not enc_r_flag  
                
            #Time sleep
            if f:
                time.sleep(1/f)
    def rad_to_arc(self,rad):
        return float(rad)/config.hardware.c_radarc
        
        