#!/usr/bin/python
import RPi.GPIO as GPIO
import smbus
import time
import math
import sys



class hmc5883l:

    __scales = {
        0.88: [0, 0.73],
        1.30: [1, 0.92],
        1.90: [2, 1.22],
        2.50: [3, 1.52],
        4.00: [4, 2.27],
        4.70: [5, 2.56],
        5.60: [6, 3.03],
        8.10: [7, 4.35],
    }

    def __init__(self, port=1, address=0x1E, gauss=1.3, declination=(0,0)):
        self.bus = smbus.SMBus(port)
        self.address = address

        (degrees, minutes) = declination
        self.__declDegrees = degrees
        self.__declMinutes = minutes
        self.__declination = (degrees + minutes / 60) * math.pi / 180

        (reg, self.__scale) = self.__scales[gauss]
        self.bus.write_byte_data(self.address, 0x00, 0x70) # 8 Average, 15 Hz, normal measurement
        self.bus.write_byte_data(self.address, 0x01, reg << 5) # Scale
        self.bus.write_byte_data(self.address, 0x02, 0x00) # Continuous measurement

    def declination(self):
        return (self.__declDegrees, self.__declMinutes)

    def twos_complement(self, val, len):
        # Convert twos compliment to integer
        if (val & (1 << len - 1)):
            val = val - (1<<len)
        return val

    def __convert(self, data, offset):
        val = self.twos_complement(data[offset] << 8 | data[offset+1], 16)
        if val == -4096: return None
        return round(val * self.__scale, 4)

    def axes(self):
        data = self.bus.read_i2c_block_data(self.address, 0x00)
        #print map(hex, data)
        x = self.__convert(data, 3)
        y = self.__convert(data, 7)
        z = self.__convert(data, 5)
        return (x,y,z)

    def heading(self):
        (x, y, z) = self.axes()
        headingRad = math.atan2(y, x)
        headingRad += self.__declination

        # Correct for reversed heading
        if headingRad < 0:
            headingRad += 2 * math.pi

        # Check for wrap and compensate
        elif headingRad > 2 * math.pi:
            headingRad -= 2 * math.pi

        # Convert to degrees from radians
        headingDeg = headingRad * 180 / math.pi
        return headingDeg

    def degrees(self, headingDeg):
        degrees = math.floor(headingDeg)
        minutes = round((headingDeg - degrees) * 60)
        return (degrees, minutes)

    def __str__(self):
        (x, y, z) = self.axes()
        return "Axis X: " + str(x) + "\n" \
               "Axis Y: " + str(y) + "\n" \
               "Axis Z: " + str(z) + "\n" \
               "Declination: " + self.degrees(self.declination()) + "\n" \
               "Heading: " + self.degrees(self.heading()) + "\n"










#Distance
def measure_us(trigger,echo):    
    #Send signal
    GPIO.output(trigger, True)    
    #Sleep to settle
    time.sleep(0.001)
    #Stop signal
    GPIO.output(trigger, False)
    #Start time    
    start = time.time()
    #Measure the length of echo
    while GPIO.input(echo)==0:
        start = time.time()
    
    while GPIO.input(echo)==1:
        stop = time.time()
        
    return (stop-start)*34320/2



#Compass functions
def read_byte(port,adr):
    return bus.read_byte_data(port, adr)

def read_word(port,adr):
    high = bus.read_byte_data(port, adr)
    low = bus.read_byte_data(port, adr+1)
    val = (high << 8) + low
    return val

def read_word_2c(port,adr):
    val = read_word(port,adr)
    if (val >= 0x8000):
        return -((65535 - val) + 1)
    else:
        return val

def write_byte(port,adr, value):
    bus.write_byte_data(port,adr, value)

#Motor functions
def move_fw_start():
    global straight_duty
    
    GPIO.output(motor_left_fw, GPIO.HIGH)
    GPIO.output(motor_right_fw, GPIO.HIGH)
    
    print "GPIO is high"

    p_left.start(straight_duty)
    p_right.start(straight_duty)

def move_fw_end():
    
    p_left.stop()
    p_right.stop()
    GPIO.output(motor_left_fw, GPIO.LOW)
    GPIO.output(motor_right_fw, GPIO.LOW)
    
def move_bw_start():
    global straight_duty
    
    GPIO.output(motor_left_bw, GPIO.HIGH)
    GPIO.output(motor_right_bw, GPIO.HIGH)
    
    print "GPIO is high"

    p_left.start(straight_duty)
    p_right.start(straight_duty)
    
    
def move_bw_end():
    
    p_left.stop()
    p_right.stop()
    
    GPIO.output(motor_left_bw, GPIO.LOW)
    GPIO.output(motor_right_bw, GPIO.LOW)

    
    
    
def turn_right_start():
    global turn_duty,status
    
    if status!="turn_right":
        GPIO.output(motor_left_fw, GPIO.HIGH)
        GPIO.output(motor_right_bw, GPIO.HIGH)
        
        print "GPIO is high"
    
        p_left.start(turn_duty)
        p_right.start(turn_duty)
        status="turn_right"
    
def turn_right_end():
    
    global straight_duty,status
    
    if status=="turn_right":
        p_left.stop()
        p_right.stop()
        GPIO.output(motor_left_fw, GPIO.LOW)
        GPIO.output(motor_right_bw, GPIO.LOW)
        status=False
    
def turn_left_start():
    global turn_duty
    
    GPIO.output(motor_left_bw, GPIO.HIGH)
    GPIO.output(motor_right_fw, GPIO.HIGH)
    
    print "GPIO is high"

    p_left.start(turn_duty)
    p_right.start(turn_duty)
    
def turn_left_end():
    p_left.stop()
    p_right.stop()
    GPIO.output(motor_left_bw, GPIO.LOW)
    GPIO.output(motor_right_fw, GPIO.LOW)


def terminate():
    print "Terminating.."
    GPIO.cleanup()
    sys.exit()
    






#!/usr/bin/python
import smbus
import time
import math

bus = smbus.SMBus(1)
address = 0x1e

def read_byte_c(adr):
    bus = smbus.SMBus(1)
    address = 0x1e
    return bus.read_byte_data(address, adr)

def read_word_c(adr):
    bus = smbus.SMBus(1)
    address = 0x1e
    high = bus.read_byte_data(address, adr)
    low = bus.read_byte_data(address, adr+1)
    val = (high << 8) + low
    return val

def read_word_2c_c(adr):
    val = read_word_c(adr)
    if (val >= 0x8000):
        return -((65535 - val) + 1)
    else:
        return val

def write_byte_c(adr, value):
    bus = smbus.SMBus(1)
    address = 0x1e
    bus.write_byte_data(address, adr, value)


def get_angle():

    write_byte_c(0, 0b00000000) 
    write_byte_c(1, 0b1111111) 
    write_byte_c(2, 0b00000000)
    scale = 0.92
    x_out = read_word_2c_c(3) * scale
    y_out = read_word_2c_c(7) * scale
    #z_out = read_word_2c_c(5) * scale
    
    bearing  = math.atan2(y_out, x_out) 
    if (bearing < 0):
        bearing += 2 * math.pi
    
    #print "Bearing: ", math.degrees(bearing)
    return math.degrees(bearing)
    
    
#########################################
#Motor Initializing
#Board layout
GPIO.setmode(GPIO.BOARD)

#Right motor pins
motor_right_bw=12
motor_right_fw=11
motor_right_pwm=16

#Left motor pins
motor_left_bw=15
motor_left_fw=13
motor_left_pwm=18

#Ultrasonic distance
# Left sensor
trigger_left = 29
echo_left = 31
#Right sensor
trigger_right = 22
echo_right = 32 

#Define the pins
#Right motor
GPIO.setup(motor_right_fw, GPIO.OUT)
GPIO.setup(motor_right_bw, GPIO.OUT)
GPIO.setup(motor_right_pwm, GPIO.OUT)
#Left motor
GPIO.setup(motor_left_fw, GPIO.OUT)
GPIO.setup(motor_left_bw, GPIO.OUT)
GPIO.setup(motor_left_pwm, GPIO.OUT)

#Both motors are off
GPIO.output(motor_right_fw, GPIO.LOW)
GPIO.output(motor_right_bw, GPIO.LOW)
GPIO.output(motor_left_fw, GPIO.LOW)
GPIO.output(motor_left_bw, GPIO.LOW)

# Set ultrasonic pins as output and input
GPIO.setup(trigger_left,GPIO.OUT)  # Trigger
GPIO.setup(trigger_right,GPIO.OUT)  # Trigger
GPIO.setup(echo_left,GPIO.IN)      # Echo
GPIO.setup(echo_right,GPIO.IN)      # Echo




freq=100
straight_duty=80
turn_duty=50

status=False

p_left = GPIO.PWM(motor_left_pwm, freq)
p_right = GPIO.PWM(motor_right_pwm, freq)





bus = smbus.SMBus(1) # or bus = smbus.SMBus(1) for Revision 2 boards

#Compass initializing
compass_port = 0x1e

#Acc initializing
# Power management registers
power_mgmt_1 = 0x6b
power_mgmt_2 = 0x6c


acc_port = 0x68       # This is the address value read via the i2cdetect command

# Now wake the 6050 up as it starts in sleep mode
bus.write_byte_data(acc_port, power_mgmt_1, 0)




compass = hmc5883l(gauss = 4.7, declination = (-2,5))  
# Script

#time.sleep(1)
while True:
    
    #time.sleep(0.1)

    accel_xout = read_word_2c(acc_port,0x3b)
    #print accel_xout
    #angle=get_angle()
    #print angle
    #sys.stdout.write("\rHeading: " + str(compass.degrees(compass.heading())) + "     ")
    angle=compass.degrees(compass.heading())[0]
    #print angle
    
    #sys.stdout.flush()
    #time.sleep(0.1)
    #accel_xout = 2

    
    ######################
    #Distance measurement
    # Set trigger to False (Low)
    #GPIO.output(trigger_left, False)
    #GPIO.output(trigger_right, False)
    # Allow module to settle
    #time.sleep(0.5)
    # Calculate pulse length
    #distance_1 = measure_us(trigger_left,echo_left)
    #distance_2 = measure_us(trigger_left,echo_left)
    distance_1=1
    distance_2=2
    print angle, "\t",accel_xout,"\t",distance_1,"\t",distance_2
    #elapsed_right = measure_us(trigger_right,echo_right)
    # Distance pulse travelled in that time is time
    # multiplied by the speed of sound (cm/s)
    #distance_left = elapsed_left*34320/2
    #distance_right=elapsed_right*34320/2
    #print "Accel-x ",accel_xout," Compass ",angle," Distance left ",distance_left," distance right ",distance_right
    #turn_right_stop()
    turn_right_start()
#         


